package org.seasar.javelin;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * JavelinŜ̐ݒێNXB(Singleton)
 *
 * l߂Ď擾悤ƂƂigetter\bh߂ČĂ񂾂Ƃj
 * t@Cݒ[hB
 *
 * @author sakamoto
 */
public class JavelinConfigUtil
{
    private static final String JAVELIN_OPTION_KEY = S2JavelinConfig.JAVELIN_PREFIX + "property";

    /** ݒt@C */
    private String fileName_;

    private Properties properties_;

    private static JavelinConfigUtil configUtil_ = new JavelinConfigUtil();


    /**
     * Singleton
     */
    private JavelinConfigUtil()
    {
        this.fileName_ = System.getProperty(JAVELIN_OPTION_KEY);
    }


    /**
     * JavelinConfigUtil̃CX^XԂB
     *
     * @return CX^X(Singleton)
     */
    public static JavelinConfigUtil getInstance()
    {
        return configUtil_;
    }


    /**
     * ݒt@CǂݍށB
     */
    private void load()
    {
        this.properties_ = new Properties();
        if (this.fileName_ != null)
        {
            try
            {
                File file = new File(this.fileName_);
                FileInputStream stream = new FileInputStream(file);
                this.properties_.load(stream);
                stream.close();

                // ݒt@Ci*.confĵfBNg擾
                File optionFile = new File(this.fileName_);
                File optionPath = optionFile.getParentFile();
                if (optionPath != null)
                {
                    this.properties_.setProperty(JAVELIN_OPTION_KEY, optionPath.getAbsolutePath());
                }
            }
            catch (IOException e)
            {
                System.err.println("IvVt@C̓ǂݍ݂Ɏs܂B"
                                   + "(" + this.fileName_ + ")");
            }
        }
        else
        {
            System.err.println("Kvȁi*.confjt@Cw肳Ă܂B");
        }
    }


    /**
     * w肳ꂽL[ɑΉ镶ԂB
     *
     * @param key L[
     * @param defaultValue ftHgl
     * @return l
     */
    public String getString(String key, String defaultValue)
    {
        synchronized (this)
        {
            if (this.properties_ == null)
            {
                load();
            }
        }
        String value = this.properties_.getProperty(key);
        if (value == null)
        {
            value = defaultValue;
        }
        return value;
    }


    /**
     * w肳ꂽL[ɑΉ鐔lԂB
     *
     * @param key L[
     * @param defaultValue ftHgl
     * @return l
     * @throws NumberFormatException l̕񂪐l\ĂȂ
     */
    public int getInteger(String key, int defaultValue)
    {
        String value = getString(key, null);
        if (value != null)
        {
            try
            {
                return Integer.parseInt(value);
            }
            catch (NumberFormatException nfe)
            {
                throw new NumberFormatException(value);
            }
        }
        return defaultValue;
    }


    /**
     * w肳ꂽL[ɑΉ鐔lԂB
     *
     * @param key L[
     * @param defaultValue ftHgl
     * @return l
     * @throws NumberFormatException l̕񂪐l\ĂȂ
     */
    public long getLong(String key, long defaultValue)
    {
        String value = getString(key, null);
        if (value != null)
        {
            try
            {
                return Long.parseLong(value);
            }
            catch (NumberFormatException nfe)
            {
                throw new NumberFormatException(value);
            }
        }
        return defaultValue;
    }


    /**
     * w肳ꂽL[ɑΉBooleanlԂB
     *
     * @param key L[
     * @param defaultValue ftHgl
     * @return l
     * @throws NumberFormatException l̕truełfalsełȂꍇ
     */
    public boolean getBoolean(String key, boolean defaultValue)
    {
        String value = getString(key, null);
        if (value != null)
        {
            if ("true".equals(value))
            {
                return true;
            }
            if ("false".equals(value))
            {
                return false;
            }
            throw new NumberFormatException(value);
        }
        return defaultValue;
    }


    /**
     * w肳ꂽL[ɕZbg܂B
     *
     * @param key L[
     * @param value l
     */
    public void setString(String key, String value)
    {
        synchronized (this)
        {
            if (this.properties_ == null)
            {
                load();
            }
        }
        this.properties_.setProperty(key, value);
    }


    /**
     * w肳ꂽL[ɐlZbg܂B
     *
     * @param key L[
     * @param value l
     */
    public void setInteger(String key, int value)
    {
        String valueString = String.valueOf(value);
        setString(key, valueString);
    }


    /**
     * w肳ꂽL[ɐlZbg܂B
     *
     * @param key L[
     * @param value l
     */
    public void setLong(String key, long value)
    {
        String valueString = String.valueOf(value);
        setString(key, valueString);
    }


    /**
     * w肳ꂽL[booleanlZbg܂B
     *
     * @param key L[
     * @param value l
     */
    public void setBoolean(String key, boolean value)
    {
        String valueString = String.valueOf(value);
        setString(key, valueString);
    }


    /**
     * w肳ꂽL[ݒɑ݂邩ǂ𒲂ׂ܂B
     *
     * @param key L[
     * @return ݂Ȃtrue
     */
    public boolean isKeyExist(String key)
    {
        synchronized (this)
        {
            if (this.properties_ == null)
            {
                load();
            }
        }
        return this.properties_.containsKey(key);
    }


    /**
     * Configt@CԂB
     *
     * @return t@C
     */
    public String getFileName()
    {
        return this.fileName_;
    }

}
