package org.seasar.javelin;

import java.text.SimpleDateFormat;

import org.seasar.javelin.bean.Invocation;
import org.seasar.javelin.util.StatsUtil;

public class JavelinLogMaker
{
    /**
     * ŁA샍O̎ނʂ邽߂Ɏgp"Call"IDB <br>
     */
    public static final int       ID_CALL       = 0;

    /**
     * ŁA샍O̎ނʂ邽߂Ɏgp"Return"IDB <br>
     */
    public static final int       ID_RETURN     = 1;

    /**
     * ŁA샍O̎ނʂ邽߂Ɏgp"Read"IDB <br>
     */
    public static final int       ID_READ       = 2;

    /**
     * ŁA샍O̎ނʂ邽߂Ɏgp"Write"IDB <br>
     */
    public static final int       ID_WRITE      = 3;

    /**
     * ŁA샍O̎ނʂ邽߂Ɏgp"Throw"IDB <br>
     */
    public static final int       ID_THROW      = 4;

    private static final String[] MESSAGE_TYPES = new String[]{"Call  ", "Return", "Read  ",
            "Write ", "Throw "                  };

    private static final String   NEW_LINE      = "\r\n";

    /**
     * 샍Oõ͓tH[}bgB
     */
    private static final String   DATE_PATTERN  = "yyyy/MM/dd HH:mm:ss.SSS";

    public static String createJavelinLog(int messageType, long time, CallTree tree,
            CallTreeNode node)
    {
        if (time == 0l)
        {
            return null;
        }

        CallTreeNode parent = node.getParent();
        S2JavelinConfig config = new S2JavelinConfig();
        boolean isReturnDetail = config.isReturnDetail();

        StringBuffer jvnBuffer = new StringBuffer();

        Invocation callee = node.getInvocation();
        Invocation caller;
        if (parent == null)
        {
            caller = new Invocation(null, null, tree.getRootCallerName(), "unknown", 0, 0, 0, 0);
        }
        else
        {
            caller = parent.getInvocation();
        }

        if (callee == null)
        {
            return "";
        }

        jvnBuffer.append(MESSAGE_TYPES[messageType]);

        SimpleDateFormat dateFormat = new SimpleDateFormat(DATE_PATTERN);
        jvnBuffer.append(",");
        jvnBuffer.append(dateFormat.format(time));

        if (messageType == ID_THROW)
        {
            // ONX
            addToJvnBuffer(node.getThrowable().getClass().getName(), jvnBuffer);

            // OIuWFNgID
            addToJvnBuffer(StatsUtil.getObjectID(node.getThrowable()), jvnBuffer);
        }

        // Ăяo惁\bh
        addToJvnBuffer(callee.getMethodName(), jvnBuffer);

        // ĂяoNX
        addToJvnBuffer(callee.getClassName(), jvnBuffer);

        // TODO ĂяoIuWFNgID
        addToJvnBuffer("unknown", jvnBuffer);

        if (messageType == ID_READ || messageType == ID_WRITE)
        {
            // TODO ANZX\bh
            addToJvnBuffer("", jvnBuffer);

            // TODO ANZXNX
            addToJvnBuffer("", jvnBuffer);

            // TODO ANZXIuWFNgID
            addToJvnBuffer("", jvnBuffer);

            // TODO ANZXtB[ȟ^
            addToJvnBuffer("", jvnBuffer);
        }
        else if (messageType == ID_CALL || messageType == ID_RETURN)
        {
            // Ăяo\bh
            addToJvnBuffer(caller.getMethodName(), jvnBuffer);

            // ĂяoNX
            addToJvnBuffer(caller.getClassName(), jvnBuffer);

            // TODO ĂяoIuWFNgID
            addToJvnBuffer("unknown", jvnBuffer);

            // TODO fBt@CA
            addToJvnBuffer("", jvnBuffer);
        }

        // XbhID
        addToJvnBuffer(tree.getThreadID(), jvnBuffer);
        jvnBuffer.append(NEW_LINE);

        if (messageType == ID_CALL && node.getArgs() != null && node.getArgs().length > 0)
        {
            jvnBuffer.append("<<javelin.Args_START>>");
            jvnBuffer.append(NEW_LINE);
            for (int i = 0; i < node.getArgs().length; i++)
            {
                jvnBuffer.append("args[");
                jvnBuffer.append(i);
                jvnBuffer.append("] = ");

                jvnBuffer.append(
                		StatsUtil.toStr(node.getArgs()[i]
                		                , config.getStringLimitLength()));

                jvnBuffer.append(NEW_LINE);
            }
            jvnBuffer.append("<<javelin.Args_END>>");
            jvnBuffer.append(NEW_LINE);
        }

        if (messageType == ID_RETURN && node.getReturnValue() != null)
        {
            jvnBuffer.append("<<javelin.Return_START>>");
            jvnBuffer.append(NEW_LINE);
            if (isReturnDetail)
            {
                jvnBuffer.append(StatsUtil.toStr(node.getReturnValue(), config.getStringLimitLength()));
            }
            else
            {
                jvnBuffer.append(
                		StatsUtil.toStr(node.getReturnValue()
                				        , config.getStringLimitLength()));
            }
            jvnBuffer.append(NEW_LINE);
            jvnBuffer.append("<<javelin.Return_END>>");
            jvnBuffer.append(NEW_LINE);
        }

        if (messageType == ID_CALL && node.getStacktrace() != null)
        {
            jvnBuffer.append("<<javelin.StackTrace_START>>");
            jvnBuffer.append(NEW_LINE);
            for (int i = 0; i < node.getStacktrace().length; i++)
            {
                jvnBuffer.append(node.getStacktrace()[i]);
                jvnBuffer.append(NEW_LINE);
            }
            jvnBuffer.append("<<javelin.StackTrace_END>>");
            jvnBuffer.append(NEW_LINE);
        }

        
        if (messageType == ID_THROW)
        {
            jvnBuffer.append("<<javelin.StackTrace_START>>");
            jvnBuffer.append(NEW_LINE);
            jvnBuffer.append(node.getThrowable().getClass().getName() + ":" + node.getThrowable().getLocalizedMessage());
            jvnBuffer.append(NEW_LINE);
            StackTraceElement[] stacktrace = node.getThrowable().getStackTrace();
            for (int i = 0; i < stacktrace.length; i++)
            {
                jvnBuffer.append(stacktrace[i]);
                jvnBuffer.append(NEW_LINE);
            }
            jvnBuffer.append("<<javelin.StackTrace_END>>");
            jvnBuffer.append(NEW_LINE);
        }
        
        String jvnMessage = jvnBuffer.toString();
        return jvnMessage;
    }

    private static void addToJvnBuffer(String element, StringBuffer jvnBuffer)
    {
        jvnBuffer.append(",\"");
        jvnBuffer.append(element);
        jvnBuffer.append("\"");
    }

}
