package org.seasar.javelin;

import java.lang.management.ManagementFactory;
import java.util.List;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.seasar.javelin.bean.Component;
import org.seasar.javelin.bean.ComponentMBean;
import org.seasar.javelin.bean.Container;
import org.seasar.javelin.bean.ContainerMBean;
import org.seasar.javelin.bean.Invocation;
import org.seasar.javelin.bean.InvocationMBean;
import org.seasar.javelin.bean.Statistics;
import org.seasar.javelin.bean.StatisticsMBean;

public class JmxRecorder
{
    /** vbgtH[MBeanT[o */
    private static MBeanServer               server_     = ManagementFactory.getPlatformMBeanServer();

    /** tOBς݂̏ꍇtrueB */
    private static boolean     isInitialized_   = false;

    /**
     * B
     * MBeanServerւContainerMBean̓o^sB
     * JpHTTP|[gw肳Ăꍇ́AHttpAdaptor̐Ɠo^sB
     */
    private static void init(S2JavelinConfig config)
    {
        try
        {
            server_ = ManagementFactory.getPlatformMBeanServer();

            if (config.getHttpPort() != 0)
            {
            	try
            	{
                	Mx4JLauncher.execute(server_, config.getHttpPort());
            	}
            	catch(Exception ex)
            	{
            		ex.printStackTrace();
            	}
            }

            ContainerMBean container;
            ObjectName containerName = new ObjectName(config.getDomain()
                    + ".container:type=" + ContainerMBean.class.getName());
            if (server_.isRegistered(containerName))
            {
/*                Set beanSet = server_.queryMBeans(containerName, null);
                if (beanSet.size() > 0)
                {
                    ContainerMBean[] containers = (ContainerMBean[])beanSet.toArray(new ContainerMBean[beanSet.size()]);
                    container = containers[0];
                }*/
            }
            else
            {
                container = new Container();
                server_.registerMBean(container, containerName);
            }

            StatisticsMBean statistics;
            ObjectName statisticsName = new ObjectName(config.getDomain()
                    + ".statistics:type=" + StatisticsMBean.class.getName());
            if (server_.isRegistered(statisticsName))
            {
/*                Set beanSet = server_.queryMBeans(statisticsName, null);
                if (beanSet.size() > 0)
                {
                    StatisticsMBean[] statisticses = (StatisticsMBean[])beanSet.toArray(new StatisticsMBean[beanSet.size()]);
                    statistics = statisticses[0];
                }*/
            }
            else
            {
                statistics = new Statistics();
                server_.registerMBean(statistics, statisticsName);
            }
        }
        catch (Throwable th)
        {
        	th.printStackTrace();
        }
    }

    /**
     * OB
     *
     * @param className
     * @param methodName
     * @param config
     */
    public static void preProcess(
    		String className
    		, String methodName
    		, S2JavelinConfig config)
    {
        synchronized (JmxRecorder.class)
        {
            if (!isInitialized_)
            {
                isInitialized_ = true;
                init(config);
            }
        }

        try
        {
            Component componentBean = MBeanManager.getComponent(className);
            String name = config.getDomain() + ".component:type="
                    + ComponentMBean.class.getName() + ",class=" + className;
            ObjectName componentName = new ObjectName(name);
            if (componentBean == null)
            {
                componentBean = new Component(componentName, className);

                if (server_.isRegistered(componentName))
                {
                    server_.unregisterMBean(componentName);
                }
                server_.registerMBean(componentBean, componentName);
                MBeanManager.setComponent(className, componentBean);
            }

            Invocation invocationBean = componentBean.getInvocation(methodName);
            name = config.getDomain() + ".invocation:type="
                    + InvocationMBean.class.getName() + ",class=" + className
                    + ",method=" + methodName;
            ObjectName objName = new ObjectName(name);

            if (invocationBean == null)
            {
                invocationBean = 
                	new Invocation(
                		objName
                		, componentName
                		, className
                		, methodName
                		, config.getIntervalMax()
                		, config.getThrowableMax()
                		, config.getRecordThreshold()
                		, config.getAlarmThreshold());

                componentBean.addInvocation(invocationBean);
                if (server_.isRegistered(objName))
                {
                    server_.unregisterMBean(objName);
                }
                server_.registerMBean(invocationBean, objName);
            }
        }
        catch (Throwable th)
        {
            // zO̗Oꍇ͕WG[o͂ɏo͂ĂB
        	// (ʂɂ͓`dȂB)
        	th.printStackTrace();
        }
    }

    /**
     * ㏈i{jB
     * @param spent
     */
    public static void postProcess()
    {
        try
        {
            // Ăяo擾B
            CallTreeNode node = S2StatsJavelinRecorder.callerNode_.get();
            if (node == null)
            {
                // Ăяo񂪎擾łȂꍇ͏LZB
                // (ʃCŗOꍇ̂߁B)
                return;
            }

            CallTreeNode parent = node.getParent();
            if (parent != null)
            {
            }
            else
            {
                // [gm[hł̌oߎԂ臒l𒴂Ăꍇ́A
                // gUNVL^B
            	if (node.getAccumulatedTime() >= node.getInvocation().getAlarmThreshold())
            	{
                    sendExceedThresholdAlarm(node);
            	}
            }
        }
        catch (Throwable th)
        {
            // zO̗Oꍇ͕WG[o͂ɏo͂ĂB
        	// (ʂɂ͓`dȂB)
            th.printStackTrace();
        }
    }

	/**
     * ㏈i{sjB
     *
     * @param cause
     */
    public static void postProcess(Throwable cause)
    {
    }

    /**
     * 
     * @param node
     */
    private static void sendExceedThresholdAlarm(CallTreeNode node)
	{
        Invocation invocation = node.getInvocation();
        if (invocation != null)
        {
            JmxListener jmxListener = new JmxListener();
            jmxListener.sendExceedThresholdAlarm(invocation);
        }
        
        List children = node.getChildren();
		for (int index = 0; index <  children.size(); index++)
        {
			CallTreeNode child = (CallTreeNode) children.get(index);
        	sendExceedThresholdAlarm(child);
        }
	}
}
