package org.seasar.javelin;

import java.lang.management.ManagementFactory;
import java.util.ArrayList;
import java.util.List;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.seasar.javelin.bean.Component;
import org.seasar.javelin.bean.ComponentMBean;
import org.seasar.javelin.bean.Container;
import org.seasar.javelin.bean.ContainerMBean;
import org.seasar.javelin.bean.Invocation;
import org.seasar.javelin.bean.InvocationMBean;
import org.seasar.javelin.communicate.JavelinAcceptThread;
import org.seasar.javelin.util.ObjectNameUtil;
import org.seasar.javelin.util.StatsUtil;

/**
 * \bhĂяo̎擾sNXB
 */
public class S2StatsJavelinRecorder
{
    /** tO */
    private static boolean                  isInitialized;

    /** vbgtH[MBeanServer */
    private static MBeanServer              server_     = ManagementFactory.getPlatformMBeanServer();

    /**
     * \bhR[c[̋L^pIuWFNgB
     */
    public static ThreadLocal<CallTree>     callTree_   = new ThreadLocal<CallTree>() {
                                                            protected synchronized CallTree initialValue()
                                                            {
                                                                return null;
                                                            }
                                                        };

    /**
     * \bȟĂяoIuWFNgB
     */
    public static ThreadLocal<CallTreeNode> callerNode_ = new ThreadLocal<CallTreeNode>() {
                                                            protected synchronized CallTreeNode initialValue()
                                                            {
                                                                return null;
                                                            }
                                                        };

    /**
     * B MBeanServerւContainerMBean̓o^sB
     * JpHTTP|[gw肳Ăꍇ́AHttpAdaptor̐Ɠo^sB
     */
    private static void javelinInit(S2JavelinConfig config, MBeanServer server)
    {
        try
        {
            // G[K[B
            JavelinErrorLogger.initErrorLog(config);

            server = ManagementFactory.getPlatformMBeanServer();

            ContainerMBean container;
            ObjectName containerName = new ObjectName(config.getDomain() + ".container:type="
                    + ContainerMBean.class.getName());
            if (server.isRegistered(containerName))
            {
                /**
                 * svȑȂ̂ŁA폜 Set beanSet = server_.queryMBeans(containerName,
                 * null); if (beanSet.size() > 0) { ContainerMBean[] containers =
                 * (ContainerMBean[]) beanSet .toArray(new
                 * ContainerMBean[beanSet.size()]); container = containers[0]; }
                 */
            }
            else
            {
                container = new Container();
                server.registerMBean(container, containerName);
            }

            // TCPł̐ڑtJnB
            int port = config.getAcceptPort();
            JavelinAcceptThread.getInstance().start(port);

        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    /**
     * MBeanɓo^镶𐶐
     * 
     * @param className
     *            NX
     * @param methodName
     *            \bh
     * @param config
     *            ݒt@Cǂݍ񂾐ݒl
     * @return o^镶
     */
    private static String createInvocationBeanName(String className, String methodName,
            S2JavelinConfig config)
    {
        return config.getDomain() + ".invocation:type=" + InvocationMBean.class.getName()
                + ",class=" + ObjectNameUtil.encode(className) + ",method="
                + ObjectNameUtil.encode(methodName);
    }

    private static String createComponentBeanName(String className, S2JavelinConfig config)
    {
        return config.getDomain() + ".component:type=" + ComponentMBean.class.getName() + ",class="
                + ObjectNameUtil.encode(className);
    }

    /**
     * JavelinRecorder, JDBCJavelinRecorderĂяoƂ̑OB
     * 
     * @param className
     * @param methodName
     * @param config
     * @param args
     */
    public static void preProcess(String className, String methodName, Object[] args,
            S2JavelinConfig config)
    {
        synchronized (S2StatsJavelinRecorder.class)
        {
            // 
            if (isInitialized == false)
            {
                javelinInit(config, server_);
                isInitialized = true;
            }
        }

        try
        {
            // [gNX
            StringBuffer buffer = new StringBuffer();
            buffer.append(className);
            String rootClassName = buffer.toString();

            Component componentBeanRoot = MBeanManager.getComponent(rootClassName);

            String name = createComponentBeanName(className, config);
            ObjectName componentName = new ObjectName(name);

            // [gNXꍇA[gNXݒ肷
            CallTreeNode node = S2StatsJavelinRecorder.callerNode_.get();
            if ((node == null) || !(componentBeanRoot == null))
            {
                className = rootClassName;
            }

            Component componentBean = MBeanManager.getComponent(className);
            if (componentBean == null)
            {
                componentBean = new Component(componentName, className);
                if (server_.isRegistered(componentName))
                {
                    server_.unregisterMBean(componentName);
                }
                server_.registerMBean(componentBean, componentName);

                MBeanManager.setComponent(className, componentBean);
            }

            Invocation invocation = componentBean.getInvocation(methodName);
            name = createInvocationBeanName(className, methodName, config);
            ObjectName objName = new ObjectName(name);

            if (invocation == null)
            {
                invocation = new Invocation(objName, componentName, className, methodName,
                                            config.getIntervalMax(), config.getThrowableMax(),
                                            config.getRecordThreshold(), config.getAlarmThreshold());

                componentBean.addInvocation(invocation);

                if (server_.isRegistered(objName))
                {
                    server_.unregisterMBean(objName);
                }
                server_.registerMBean(invocation, objName);
            }

            // Ăяo̓R[c[B
            CallTree tree = callTree_.get();
            if (tree == null)
            {
                tree = new CallTree();
                tree.setRootCallerName(config.getRootCallerName());
                tree.setEndCalleeName(config.getEndCalleeName());
                callTree_.set(tree);
            }

            setNode(node, tree, null, args, invocation, config);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }

    }

    /**
     * OB
     * 
     * @param className
     *            NX
     * @param methodName
     *            \bh
     * @param args
     *            
     * @param stacktrace
     *            X^bNg[X
     * @param config
     *            ݒ
     */
    public static void preProcess(String className, String methodName, Object[] args,
            StackTraceElement[] stacktrace, S2JavelinConfig config)
    {
        synchronized (S2StatsJavelinRecorder.class)
        {
            // 
            if (isInitialized == false)
            {
                javelinInit(config, server_);
                isInitialized = true;
            }
        }

        Component component = MBeanManager.getComponent(className);
        Invocation invocation = component.getInvocation(methodName);

        try
        {
            // Ăяo擾B
            CallTreeNode node = callerNode_.get();
            CallTree tree = null;

            if (node == null)
            {
                // Ăяo̓R[c[B
                tree = callTree_.get();
                if (tree == null)
                {
                    tree = new CallTree();
                    callTree_.set(tree);
                }
                tree.setRootCallerName(config.getRootCallerName());
                tree.setEndCalleeName(config.getEndCalleeName());

                switch (config.getThreadModel())
                {
                case S2JavelinConfig.TM_THREAD_ID:
                    tree.setThreadID("" + Thread.currentThread().getId());
                    break;
                case S2JavelinConfig.TM_THREAD_NAME:
                    tree.setThreadID(Thread.currentThread().getName());
                    break;
                case S2JavelinConfig.TM_CONTEXT_PATH:
                    tree.setThreadID(methodName);
                    break;
                default:
                    break;
                }
            }
            setNode(node, tree, stacktrace, args, invocation, config);
        }
        catch (Exception ex)
        {
            // zO̗Oꍇ͕WG[o͂ɏo͂ĂB
            ex.printStackTrace();
        }
    }

    /**
     * m[hݒ肷
     * 
     * @param node
     */
    private static void setNode(CallTreeNode node, CallTree tree, StackTraceElement[] stacktrace,
            Object[] args, Invocation invocation, S2JavelinConfig config)
    {
        try
        {
            if (node == null)
            {
                // Ăяo̓R[c[B
                if (tree == null)
                {
                    tree = new CallTree();
                    tree.setRootCallerName(config.getRootCallerName());
                    tree.setEndCalleeName(config.getEndCalleeName());
                    callTree_.set(tree);
                }

                node = new CallTreeNode();
                node.setStartTime(System.currentTimeMillis());
                if (config.isLogStacktrace())
                {
                    node.setStacktrace(stacktrace);
                }
                if (tree != null)
                {
                    tree.setRootNode(node);
                }
            }
            else
            {
                // 2ڈȍ~́AKw\`
                CallTreeNode parent = node;
                node = new CallTreeNode();
                node.setStartTime(System.currentTimeMillis());
                if (config.isLogStacktrace())
                {
                    node.setStacktrace(stacktrace);
                }
                parent.addChild(node);
                invocation.addCaller(parent.getInvocation());
            }
            // p[^ݒ肪sĂƂAm[hɃp[^ݒ肷
            if (config.isLogArgs())
            {
                String[] argStrings = new String[args.length];
                for (int index = 0; index < args.length; index++)
                {
                    if (config.isArgsDetail())
                    {
                        int argsDetailDepth = config.getArgsDetailDepth();
                        argStrings[index] = StatsUtil.objectDetailPrinter(args[index],
                                                                          argsDetailDepth, 0);
                    }

                    argStrings[index] = StatsUtil.toStr(args[index], config.getStringLimitLength());
                }
                node.setArgs(argStrings);
            }

            node.setInvocation(invocation);

            // ĂяoA
            // 񃍃Oǒ͎ĂяoƂĎgp邽߂ɕۑB
            callerNode_.set(node);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    /**
     * ㏈i{jB
     * 
     * @param returnValue
     *            ߂l
     * @param config
     *            ݒ
     */
    public static void postProcess(Object returnValue, S2JavelinConfig config)
    {
        try
        {
            // Ăяo擾B
            CallTreeNode node = callerNode_.get();
            if (node == null)
            {
                // Ăяo񂪎擾łȂꍇ͏LZB
                // (ʃCŗOꍇ̂߁B)
                return;
            }

            node.setEndTime(System.currentTimeMillis());
            if (returnValue != null)
            {
                String returnString;
                if (config.isReturnDetail())
                {
                    int returnDetailDepth = config.getReturnDetailDepth();
                    returnString = StatsUtil.objectDetailPrinter(returnValue, returnDetailDepth, 0);
                    returnString = StatsUtil.toStr(returnString, config.getStringLimitLength());
                }
                else
                {
                    returnString = StatsUtil.toStr(returnValue, config.getStringLimitLength());
                }
                node.setReturnValue(returnString);
            }

            CallTreeNode parent = node.getParent();
            if (parent != null)
            {
                callerNode_.set(parent);
            }
            else
            {
                // vlL^臒l𒴂ĂꍇɁAgUNVL^B
                if (node.getAccumulatedTime() >= config.getStatisticsThreshold())
                {
                    recordTransaction(node);
                }

                // t@Co͂臒l𒴂ĂꍇɁAJavelinOt@Cɏo͂B
                if (node.getAccumulatedTime() >= node.getInvocation().getRecordThreshold())
                {
                    S2StatsJavelinFileGenerator generator = new S2StatsJavelinFileGenerator(
                                                                                            config.getJavelinFileDir());
                    generator.generateJaveinFile(callTree_.get(), node);
                }

                // A[臒l𒴂ĂꍇɁAA[ʒmB
                if (node.getAccumulatedTime() >= node.getInvocation().getAlarmThreshold())
                {
                    sendExceedThresholdAlarm(node);
                }

                callerNode_.set(null);
            }
        }
        catch (Exception ex)
        {
            // zO̗Oꍇ͕WG[o͂ɏo͂ĂB
            ex.printStackTrace();
        }
    }

    private static Throwable cause_;

    /**
     * ㏈i{sjB
     * 
     * @param cause
     */
    public static void postProcess(Throwable cause, S2JavelinConfig config)
    {
        try
        {
            // Ăяo擾B
            CallTreeNode node = callerNode_.get();

            if (node == null)
            {
                // Ăяo񂪎擾łȂꍇ͏LZB
                // (ʃCŗOꍇ̂߁B)
                return;
            }

            if (cause_ != cause)
            {
                node.setThrowable(cause);
                node.setThrowTime(System.currentTimeMillis());
            }

            CallTreeNode parent = node.getParent();
            if (parent != null)
            {
                callerNode_.set(parent);
            }
            else
            {
                recordTransaction(node);

                // JavelinOt@Co͂B
                S2StatsJavelinFileGenerator generator = new S2StatsJavelinFileGenerator(
                                                                                        config.getJavelinFileDir());
                CallTreeNode callTreeNode = callerNode_.get();
                CallTree callTree = callTree_.get();
                generator.generateJaveinFile(callTree, callTreeNode);

                // A[𑗐MB
                sendExceedThresholdAlarm(node);

                callerNode_.set(null);
            }

            if (cause_ == cause)
            {
                // Ăяo񂪎擾łȂꍇ͏LZB
                // (łɋL^ς݂̗Ô߁B)
                return;
            }

            // OL^ĂB
            node.getInvocation().addThrowable(cause);

            cause_ = cause;
        }
        catch (Exception ex)
        {
            // zO̗Oꍇ͕WG[o͂ɏo͂ĂB
            ex.printStackTrace();
        }
    }

    public static void dumpJavelinLog(S2JavelinConfig config)
    {
        // JavelinOt@Co͂B
        S2StatsJavelinFileGenerator generator = new S2StatsJavelinFileGenerator(
                                                                                config.getJavelinFileDir());
        
        
        CallTree callTree = callTree_.get();
        CallTreeNode root = callTree.getRootNode();
        if (root != null && callTree != null)
        {
            generator.generateJaveinFile(callTree, root);

        }
    }

    public static void preProcessField(String className, String methodName, S2JavelinConfig config)
    {
        synchronized (S2StatsJavelinRecorder.class)
        {
            // 
            if (isInitialized == false)
            {
                javelinInit(config, server_);
                isInitialized = true;
            }
        }

        try
        {
            // Ăяo擾B
            CallTreeNode node = callerNode_.get();
            CallTree tree = null;

            if (node == null)
            {
                // Ăяo̓R[c[B
                tree = callTree_.get();
                if (tree == null)
                {
                    tree = new CallTree();
                    callTree_.set(tree);
                }
                tree.setRootCallerName(config.getRootCallerName());
                tree.setEndCalleeName(config.getEndCalleeName());
            }

            CallTreeNode parent = node;
            node = new CallTreeNode();
            node.setStartTime(System.currentTimeMillis());
            node.setFieldAccess(true);
            Invocation invocation = parent.getInvocation();
            node.setInvocation(invocation);
            parent.addChild(node);
            // ĂяoA񃍃Oǒ͎ĂяoƂĎgp邽߂ɕۑB
            callerNode_.set(node);
        }
        catch (Exception ex)
        {
            // zO̗Oꍇ͕WG[o͂ɏo͂ĂB
            ex.printStackTrace();
        }
    }

    public static void postProcessField(S2JavelinConfig config)
    {
        try
        {
            // Ăяo擾B
            CallTreeNode node = callerNode_.get();
            if (node == null)
            {
                // Ăяo񂪎擾łȂꍇ͏LZB
                // (ʃCŗOꍇ̂߁B)
                return;
            }

            node.setEndTime(System.currentTimeMillis());

            CallTreeNode parent = node.getParent();
            if (parent != null)
            {
                callerNode_.set(parent);
            }
        }
        catch (Exception ex)
        {
            // zO̗Oꍇ͕WG[o͂ɏo͂ĂB
            ex.printStackTrace();
        }
    }

    /**
     * gUNVL^B
     * 
     * @param node
     */
    public static void recordTransaction(CallTreeNode node)
    {
        Invocation invocation = node.getInvocation();
        if (invocation != null)
        {
            invocation.addInterval(StatsUtil.getElapsedTime(node));
            if (node.getParent() != null)
            {
                invocation.addCaller(node.getParent().getInvocation());
            }
        }

        List children = node.getChildren();
        for (int index = 0; index < children.size(); index++)
        {
            CallTreeNode child = (CallTreeNode)children.get(index);
            recordTransaction(child);
        }
    }

    private static void sendExceedThresholdAlarm(CallTreeNode node)
    {
        sendExceedThreasholdAlarmImpl(node);

        List children = node.getChildren();
        for (int index = 0; index < children.size(); index++)
        {
            CallTreeNode child = (CallTreeNode)children.get(index);
            sendExceedThresholdAlarm(child);
        }
    }

    private static void sendExceedThreasholdAlarmImpl(CallTreeNode node)
    {
        Invocation invocation = node.getInvocation();

        synchronized (alarListenerList__)
        {
            for (AlarmListener alarmListener : alarListenerList__)
            {
                alarmListener.sendExceedThresholdAlarm(invocation);
            }
        }

    }

    private static final List<AlarmListener> alarListenerList__ = new ArrayList<AlarmListener>();
    static
    {
        JmxListener jmxListener = new JmxListener();
        addListener(jmxListener);

    }

    public static void addListener(AlarmListener alarmListener)
    {
        synchronized (alarListenerList__)
        {
            alarListenerList__.add(alarmListener);
        }
    }

    /**
     * \bhĂяoc[B
     * 
     * @param callTree
     *            \bhĂяoc[
     */
    public static void initCallTree(CallTree callTree)
    {
        callTree_.set(callTree);
    }

    /**
     * XbhIDݒ肷
     * 
     * @param threadId
     *            XbhID
     */
    public static void setThreadId(String threadId)
    {
        CallTree callTree = callTree_.get();
        callTree.setThreadID(threadId);
    }

}
